package com.zhilian.resolver.service.impl.eventMalfunction;

import com.alibaba.fastjson.JSON;
import com.zhilian.business.domain.BusinessBreak;
import com.zhilian.business.remote.RemoteBreakService;
import com.zhilian.common.redis.service.RedisService;
import com.zhilian.common.resolver.domain.ResolverReportData;
import com.zhilian.resolver.mapper.UserMapper;
import com.zhilian.resolver.service.ResolverEventService;
import com.zhilian.resolver.service.ResolverReportInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
  * @ClassName MalfunctionEventServiceImpl
  * @Description 故障事件服务
  * @Author Can.J
  * @Date  2024/4/4 10:00
 */
@Slf4j
@Service("malfunction")
public class MalfunctionEventServiceImpl implements ResolverEventService {
    @Autowired
    private RedisService redisService;

    /**
     *  获取用信息 模拟故障
     */
    @Autowired
    private UserMapper userMapper;

    /**
     *  注入ResolverReportInfoService 业务层
     */
    @Autowired
    private ResolverReportInfoService resolverReportInfoService;

    @Autowired
    private RemoteBreakService remoteBreakService;

    @Override
    public void execute(ResolverReportData vehicleData) {
        log.info("故障事件开始执行");
                if(isAnyFieldZero(vehicleData)){
                    resolverReportInfoService.saveDataToDatabaseByMalfunction(vehicleData);

                    BusinessBreak businessBreak = new BusinessBreak();
                    businessBreak.setBreakVin(vehicleData.getVin());
                    //     * 车辆状态 1：正常 0：故障
                    if(vehicleData.getVehicleStatus()==0){
                        businessBreak.setBreakCode("CL001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("车体故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * 充电状态 1：正常 0：故障
                    //    private int chargingStatus;
                    if(vehicleData.getChargingStatus()==0){
                        businessBreak.setBreakCode("CD001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电池故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * 运行状态 1：正常 0：故障
                    if(vehicleData.getOperatingStatus()==0){
                        businessBreak.setBreakCode("DW001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("行驶故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * SOC 状态 1：正常 0：故障
                    if(vehicleData.getSocStatus()==0){
                        businessBreak.setBreakCode("SOC001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电池故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     可充电储能装置工作状态 1：正常 0：故障;
                    if(vehicleData.getChargingEnergyStorageStatus()==0){
                        businessBreak.setBreakCode("CDC001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电池故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //      驱动电机状态 1：正常 0：故障
                    if(vehicleData.getDriveMotorStatus()==0){
                        businessBreak.setBreakCode("DJ001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电机故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     定位是否有效 1：有效 0：无效
                    if(vehicleData.getPositionStatus()==0){
                        businessBreak.setBreakCode("SOC001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("定位故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * EAS(汽车防盗系统)状态 1：正常 0：故障
                    if(vehicleData.getEasStatus()==0){
                        businessBreak.setBreakCode("EAS001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("EAS故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * PTC(电动加热器)状态 1：正常 0：故障
                    if(vehicleData.getPtcStatus()==0){
                        businessBreak.setBreakCode("PTC001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("加热故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * EPS(电动助力系统)状态 1：正常 0：故障
                    if(vehicleData.getEpsStatus()==0){
                        businessBreak.setBreakCode("EPS001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电动助力故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * ABS(防抱死)状态 1：正常 0：故障
                    if(vehicleData.getAbsStatus()==0){
                        businessBreak.setBreakCode("ABS001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("ABS故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     *  MCU(电机/逆变器)状态 1：正常 0：故障
                    if(vehicleData.getMcuStatus()==0){
                        businessBreak.setBreakCode("MCU001");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("MCU故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     *  动力电池加热状态 1：正常 0：故障
                    if(vehicleData.getHeatingStatus()==0){
                        businessBreak.setBreakCode("DC002");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电池故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * 动力电池当前状态 1：正常 0：故障
                    if(vehicleData.getBatteryStatus()==0){
                        businessBreak.setBreakCode("DC003");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电池故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     *  动力电池保温状态 1：正常 0：故障
                    if(vehicleData.getBatteryInsulationStatus()==0){
                        businessBreak.setBreakCode("DC004");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电池故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     *  DCDC(电力交换系统) 状态 1：正常 0：故障
                    if(vehicleData.getDcdcStatus()==0){
                        businessBreak.setBreakCode("DCDC005");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电力故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }

                    //     * CHG(充电机)状态 1：正常 0：故障
                    if(vehicleData.getChgStatus()==0){
                        businessBreak.setBreakCode("CDJ005");
                        businessBreak.setBreakDate(new Date());
                        businessBreak.setBreakType("电力故障");
                        businessBreak.setBreakState("1");
                        remoteBreakService.add(businessBreak);
                    }




//                    String key = "breakdown";
                    String value = JSON.toJSONString(businessBreak);
                    redisService.setCacheObject("breakdown:gz:"+vehicleData.getVin(),value);
                    long expireTime = 30;
                    redisService.expire("breakdown:gz:"+vehicleData.getVin(), expireTime, TimeUnit.MINUTES);

                }


        log.info("故障事件结束");

    }
    private boolean isAnyFieldZero(ResolverReportData resolverReportData) {
        return resolverReportData.getChargingStatus() == 0 ||
                resolverReportData.getOperatingStatus() == 0 ||
                resolverReportData.getSocStatus() == 0 ||
                resolverReportData.getVehicleStatus() == 0 ||
                resolverReportData.getChargingEnergyStorageStatus() == 0 ||
                resolverReportData.getDriveMotorStatus() == 0 ||
                resolverReportData.getPositionStatus() == 0 ||
                resolverReportData.getEasStatus() == 0 ||
                resolverReportData.getPtcStatus() == 0 ||
                resolverReportData.getEpsStatus() == 0 ||
                resolverReportData.getAbsStatus() == 0 ||
                resolverReportData.getMcuStatus() == 0 ||
                resolverReportData.getHeatingStatus() == 0 ||
                resolverReportData.getBatteryStatus() == 0 ||
                resolverReportData.getBatteryInsulationStatus() == 0 ||
                resolverReportData.getDcdcStatus() == 0 ||
                resolverReportData.getChgStatus() == 0;

    }
    @Override
    public String getEventName() {
        return "malfunction";
    }
}
